<?php
/**
 * Class that runs a tiny little game with polar-pears and water-holes :o)
 *
 * $Id: tgcPolarBears.php,v 1.4 2004/02/23 21:42:36 luckec Exp $
 *
 * @package     tgcPolarBears
 * @author      Carsten Lucke <luckec@tool-garage.de>
 * @copyright   Carsten Lucke <http://www.tool-garage.de>
 */

 
/**
 * Include directory
 *
 * @access  public
 */
define( 'TGCPOLARBEARS_INCLUDEDIR', 'include' );

/**
 * Template directory
 *
 * @access  public
 */
define( 'TGCPOLARBEARS_TEMPLATEDIR', 'templates' );

/**
 * Template directory
 *
 * @access  public
 */
define( 'TGCPOLARBEARS_STYLEDIR', 'css' );

/**
 * number of dices used in the game
 *
 * @access  public
 */
define( 'TGCPOLARBEARS_DICES_USED', 5 );

/**
 * number of dices displayed in a row
 *
 * @access  public
 */
define( 'TGCPOLARBEARS_DICES_PER_ROW', 10 );

/**
 * english-language template
 *
 * @access  public
 */
define( 'TGCPOLARBEARS_LANG_TEMPLATE_EN', 'frame_en.tmpl' );

/**
 * german-language template
 *
 * @access  public
 */
define( 'TGCPOLARBEARS_LANG_TEMPLATE_DE', 'frame_de.tmpl' );



/**
 * Uses patTemplate for visualization
 *
 * @link    http://www.php-tools.de
 */
require_once TGCPOLARBEARS_INCLUDEDIR . '/patTemplate.php';
//require_once TGCPOLARBEARS_INCLUDEDIR . DIRECTORY_SEPARATOR . 'patTemplate.php';
 

/**
 * Class that runs a tiny game with polar-pears and water-holes :o)
 *
 * @package      tgcPolarBears
 * @access       public
 * @version      1.1.1
 * @author       Carsten Lucke                   <luckec@tool-garage.de>
 */
class tgcPolarBears
{
    /**
     * contains the dices, that shall be displayed
     *
     * @access  private
     * @var     array
     */
    var $_dices;
    
    /**
     * number of bears sitting around water-holes
     *
     * @access  private
     * @var     int
     */
    var $_bears;
    
    /**
     * number of water-holes
     *
     * @access  private
     * @var     int
     */
    var $_holes;
    
    /**
     * determines whether there is fish around or not
     *
     * @access  private
     * @var     boolean
     */
    var $_fish;
    
    /**
     * template object
     *
     * @access  private
     * @var     object
     */
    var $_tmpl;
    
    /**
     * skin- and template-folder
     *
     * @access  private
     * @var     boolean
     */
    var $_skinDir = 'default';
    
    /**
     * language
     *
     * @access  private
     * @var     boolean
     */
    var $_lang = 'de';
    
    /**
     * Constructor
     *
     * @access  public
     */
    function tgcPolarBears()
    {
        $tmpl               =&  new patTemplate();
        $tmpl->setBasedir(TGCPOLARBEARS_TEMPLATEDIR);
        $this->_tmpl        =&  $tmpl;
        
        $this->_generateRandomDices();
        $this->_bears       =   $this->_countBears();
        $this->_holes       =   $this->_countHoles();
        $this->_fish        =   $this->_isFishAround();
    }
    
    /**
     * Set the skin
     *
     * @access  public
     * @param   string  $id     skin-id
     */
    function setSkin($id)
    {
        $this->_skinDir     =   $id;
    }
    
    /**
     * Set the language
     *
     * @access  public
     * @param   string  $lang     language
     */
    function setLanguage($lang)
    {
        $this->_lang        =   $lang;
    }
    
    /**
     * Returns the ASCII text for a dice
     *
     * @access  private
     * @param   int         $eyeNum     a dice's eye-number  ('ONE', 'TWO', ..., 'SIX')
     * @return   string      
     */
    function _getDiceText($eyeNum)
    {
        return $this->_tmpl->getParsedTemplate('dice_' . $this->_numberToString($eyeNum));
    }
     
    /**
     * Parses the dices that were randomly chosen into the template
     *
     * @access  private
     * @return  string      content
     */
    function _parseDices()
    {
        $this->_tmpl->clearTemplate('dices_row');
        $this->_tmpl->clearTemplate('dices_col');
        $currentCol         =   0;
        
        foreach ($this->_dices as $diceEyeNum) {
            $dice           =   $this->_getDiceText($diceEyeNum);
            $this->_tmpl->addVar('dices_col', 'DICE', $dice);
            $this->_tmpl->parseTemplate('dices_col', 'a');
            if (++$currentCol == TGCPOLARBEARS_DICES_PER_ROW) {
                $this->_tmpl->parseTemplate('dices_row', 'a');
                $this->_tmpl->clearTemplate('dices_col');
            }
        }
        
        $openCols           =   TGCPOLARBEARS_DICES_PER_ROW - $currentCol;
        for ($i = 0; $i < $openCols; $i++) {
            
            $this->_tmpl->addVar('dices_col', 'DICE', '&#160;');
            $this->_tmpl->parseTemplate('dices_col', 'a');
        }
        $this->_tmpl->parseTemplate('dices_row', 'a');
    }
    
    /**
     * Parses the dices that were randomly chosen into the template
     *
     * @access  private
     * @return  string      content
     */
    function _parseSolution()
    {
        $this->_tmpl->addVar('tgcPolarBears_page', 'NUM_BEARS', $this->_bears);
        $this->_tmpl->addVar('tgcPolarBears_page', 'NUM_HOLES', $this->_holes);
        $this->_tmpl->addVar('fish_around', 'IS_FISH_AROUND', $this->_fish ? 'yes' : 'no');
    }
    
    /**
     * Generates randomly chosen dices/eye numbers
     *
     * @access  private
     */
    function _generateRandomDices()
    {
        $this->_dices       =   array();
        srand((double) microtime() * 1000000);
        
        for ($i = 0; $i < TGCPOLARBEARS_DICES_USED; $i++)
        {
            $randEyeNum     =   rand(1, 6);
            array_push($this->_dices, $randEyeNum);
        }
    }
    
    /**
     * Converts a number (1-6) into a string (one - six)
     *
     * @access  private
     * @param   int     $number     number
     * @param   boolean $toUpper    upper- or lowercase
     * @return  string  number
     */
    function _numberToString($number, $toUpper = true)
    {
        $strNum             =   '';
        switch ($number) {
            case 1:
                $strNum     =   'one';
                break;
                
            case 2:
                $strNum     =   'two';
                break;
                
            case 3:
                $strNum     =   'three';
                break;
                
            case 4:
                $strNum     =   'four';
                break;
                
            case 5:
                $strNum     =   'five';
                break;
                
            case 6:
                $strNum     =   'six';
                break;
        }
        if ($toUpper) {
            return strtoupper($strNum);
        }
        return $strNum;
    }
    
    /**
     * Counts the bears that are sitting around water-holes
     *
     * @access  private
     * @return  int     number of bears
     */
    function _countBears()
    {
        $numBears           =   0;
        foreach ($this->_dices as $dice) {
            switch ($dice) {
                case 3:
                    $numBears   +=  2;
                    break;
                    
                case 5:
                    $numBears   +=  4;
                    break;
            }
        }
        return $numBears;
    }
    
    /**
     * Counts the number of water-holes
     * 
     * @access  private
     * @return  int     number of water-holes
     */
    function _countHoles()
    {
        $numHoles           =   0;
        foreach ($this->_dices as $dice) {
            if ($dice == 1 || $dice == 3 || $dice == 5) {
                ++$numHoles;
            }
        }
        return $numHoles;
    }
    
    /**
     * Checks if fish is in the area
     *
     * @access  private
     * @return  boolean     true if fish is hanging around, otherwise false
     */
    function _isFishAround()
    {
        if (in_array(6, $this->_dices)) {
            return true;
        }
        return false;
    }
    
    /**
     * Starts processing the game
     *
     * @access  public
     */
    function runGame()
    {
        switch ($this->_lang) {
            case 'de':
                $templateFile   =   TGCPOLARBEARS_LANG_TEMPLATE_DE;
                break;
            case 'en':
                $templateFile   =   TGCPOLARBEARS_LANG_TEMPLATE_EN;
                break;
        }
        $this->_tmpl->readTemplatesFromFile($this->_skinDir . '/' . $templateFile);
        
        $this->_tmpl->addGlobalVar('STYLEDIR', TGCPOLARBEARS_STYLEDIR . '/' . $this->_skinDir);
        $this->_tmpl->addGlobalVar('SELF_SITE', $_SERVER['SCRIPT_NAME']);
        
        $this->_parseDices();
        $this->_parseSolution();
        return $this->_tmpl->getParsedTemplate('tgcPolarBears_page');
    }
}
?>